
local selectedIndex = nil;

local entries = {};
local reagents = {};

function CraftBar_InitQueue()
    CraftBarQueue = CraftBarQueue or {};

    TradeSkillFrame.queueButton = CreateFrame("Button", "CraftBarQueueButton", TradeSkillFrame, "UIPanelButtonTemplate");
    TradeSkillFrame.queueButton:SetWidth(100);
    TradeSkillFrame.queueButton:SetHeight(22);
    TradeSkillFrame.queueButton:SetPoint("RIGHT", TradeSkillFrame.FilterButton, "LEFT", -6, 0);
    TradeSkillFrame.queueButton:SetText("Queue");
    TradeSkillFrame.queueButton:SetScript("OnClick", CraftBar_ToggleQueueFrame);

    TradeSkillFrame.queueFrame = CreateFrame("Frame", "CraftBarQueueFrame", TradeSkillFrame, "CraftBarQueueFrameTemplate");
    TradeSkillFrame.queueFrame:SetSize(350, TradeSkillFrame:GetHeight());
    TradeSkillFrame.queueFrame:ClearAllPoints();
    TradeSkillFrame.queueFrame:SetPoint("TOPLEFT", TradeSkillFrame, "TOPRIGHT", 50, 0);
    
    SetPortraitToTexture(CraftBarQueueFramePortrait, "Interface\\FriendsFrame\\FriendsFrameScrollIcon" );

    -- create gui
    HybridScrollFrame_OnLoad(TradeSkillFrame.queueFrame.scrollFrame);
    TradeSkillFrame.queueFrame.scrollFrame.update = CleanUI_BattlePetGroups_OnUpdate;
    HybridScrollFrame_CreateButtons(TradeSkillFrame.queueFrame.scrollFrame, "CraftBarQueueEntryTemplate");
    
    TradeSkillFrame.queueFrame.removeButton:SetEnabled(false);
    TradeSkillFrame.queueFrame.clearButton:SetEnabled(false);
end

local function checkMerchant(reagent, count)
    if (not MerchantFrame:IsShown()) then
        return nil;
    end
    
    local name, texture, price, quantity, numAvailable, isUsable, extendedCost;
    local buyQuantity;
    for index = 1, GetMerchantNumItems(), 1 do
        name, texture, price, quantity, numAvailable, isUsable, extendedCost = GetMerchantItemInfo(index);
        if (name == reagent) then
            buyQuantity = count;

            if (numAvailable > 0 and buyQuantity > numAvailable) then
                buyQuantity = numAvailable;
            end

            return index, buyQuantity, price, extendedCost;
        end
    end

    return nil;
end

function CleanUI_BattlePetGroups_OnUpdate()
    local buttons = TradeSkillFrame.queueFrame.scrollFrame.buttons;

    local numEntries = #entries;
    local numButtons = #buttons;

    local scrollOffset = HybridScrollFrame_GetOffset(TradeSkillFrame.queueFrame.scrollFrame);
    local buttonHeight = buttons[1]:GetHeight();
    local displayedHeight = 0;

    local entry;
    local data;

    for i=1, numButtons do
        local entryIndex = i + scrollOffset;
        entry = buttons[i];

        if ( entryIndex <= numEntries ) then
            data = entries[entryIndex];

            entry:Show();
            entry.index = entryIndex;
            
            entry.data = data;
            entry.createButton.data = data;

            entry:SetText(data.name);
            entry.count:SetText("|cffffffff"..data.count.."|r");
            
            entry.icon:SetTexture(data.texture);
            entry.icon:SetTexCoord(0.1, 0.9, 0.1, 0.9);
            
            if (data.type == "CREATE") then
                entry.createButton:SetText(CREATE_PROFESSION);
                    entry.createButton.type = "CREATE";
                entry.createButton:Show();
                entry.createButton:Enable();
            elseif (data.type == "MISSING_RESOURCE") then
                entry.createButton:SetText(CREATE_PROFESSION);
                entry.createButton:Show();
                entry.createButton:Disable();
            else
                local index, buyQuantity, price, extendedCost = checkMerchant(data.name, data.count);

                if (index) then
                    entry.createButton:Show();
                    entry.createButton.type = "BUY";
                    entry.createButton.index = index;
                    entry.createButton.buyQuantity = buyQuantity;
                    entry.createButton.price = price;

                    if (price > 0) then
                        entry.createButton:SetText(GetCoinTextureString(price * buyQuantity));
                    elseif ( extendedCost and (price > 0) ) then
                        entry.createButton:SetText(price * buyQuantity.." units");
                    else
                        entry.createButton:SetText("buy");
                    end

                    entry.createButton:Enable();
                else
                    entry.createButton:Hide();
                    entry.createButton:Disable();
                end
            end

            -- this isn't a header, hide the header textures
            entry:SetNormalTexture("");
            entry:SetHighlightTexture("");

            if (entryIndex == selectedIndex) then
                -- reposition highlight frames
                TradeSkillFrame.queueFrame.scrollFrame.highlightFrame:SetParent(entry);
                TradeSkillFrame.queueFrame.scrollFrame.highlightFrame:SetAllPoints(entry);
                TradeSkillFrame.queueFrame.scrollFrame.highlightFrame:Show();
            end
        else
            entry:Hide();
        end

        displayedHeight = displayedHeight + buttonHeight;
    end

    HybridScrollFrame_Update(TradeSkillFrame.queueFrame.scrollFrame, numEntries * buttonHeight, displayedHeight);
end

-- LOCAL FUNCTIONS
local function getCreateQueue()
    local tradeSkillID, skillLineName, skillLineRank, skillLineMaxRank, skillLineModifier = C_TradeSkillUI.GetTradeSkillLine();
    CraftBarQueue[skillLineName] = CraftBarQueue[skillLineName] or {}; 
    return CraftBarQueue[skillLineName];
end

local function addReagents(recipeId, count)
    local numReagents = C_TradeSkillUI.GetRecipeNumReagents(recipeId);
    
    local reagent;
    
    for reagentIndex = 1, numReagents do
        local reagentName, reagentTexture, reagentCount, playerReagentCount = C_TradeSkillUI.GetRecipeReagentInfo(recipeId, reagentIndex);
        
        if (reagentName) then
            if (reagents[reagentName]) then
                reagents[reagentName].count = reagents[reagentName].count + reagentCount * count;
            else
                reagent = {};
                
                reagent.name = reagentName;
                reagent.texture = reagentTexture;
                reagent.count = reagentCount * count;
                reagent.itemLink = C_TradeSkillUI.GetRecipeReagentItemLink(recipeId, reagentIndex);
                
                reagents[reagentName] = reagent;
            end
        end
    end
end

local function addReagentsToQueue()
    local queue = getCreateQueue();
    
    local entry, itemCount, requiredCount;
    
    for name, data in pairs(reagents) do
        entry = {};
        
        itemCount = GetItemCount(data.itemLink, true);
        
        requiredCount = data.count - itemCount;

        if (requiredCount > 0) then
            entry.name = data.name;
            entry.texture = data.texture;
            entry.count = requiredCount;
            entry.itemLink = data.itemLink;
            entry.type = "REAGENT";
            
            tinsert(entries, 1, entry);
        end
    end
end

local function calculateQueue()
    entries = {};
    reagents = {};
    
    local skillName, skillType, numAvailable, isExpanded, altVerb, numSkillUps;
    
    -- first, add user added items
    local queue = getCreateQueue();
    
    local entry, recipeInfo;
    
    for name, data in pairs(queue) do
        entry = {};

        local recipeInfo = C_TradeSkillUI.GetRecipeInfo(data.recipeId);
        
        if (recipeInfo) then
            entry.recipeId = data.recipeId;
            entry.name = recipeInfo.name;
            entry.count = data.count;
            entry.itemLink = C_TradeSkillUI.GetRecipeItemLink(data.recipeId);
            entry.type = "CREATE";
            
            addReagents(data.recipeId, entry.count);
            
            tinsert(entries, entry);
        end
    end
    
    addReagentsToQueue();
    
    CleanUI_BattlePetGroups_OnUpdate();
    
    TradeSkillFrame.queueFrame.clearButton:SetEnabled(#entries > 0);
    
    if (#entries == 0) then
        TradeSkillFrame.queueFrame.removeButton:SetEnabled(false);
        selectedIndex = nil;
        TradeSkillFrame.queueFrame.scrollFrame.highlightFrame:Hide();
    end
    
    local tradeSkillID = C_TradeSkillUI.GetTradeSkillLine();
    SetPortraitToTexture(CraftBarQueueFramePortrait, C_TradeSkillUI.GetTradeSkillTexture(tradeSkillID));
end

local function addToQueue(recipeId, repeatCount)

    local recipeInfo = C_TradeSkillUI.GetRecipeInfo(recipeId);

    local queue = getCreateQueue();
    
    if (queue[recipeInfo.name]) then
        queue[recipeInfo.name].count = queue[recipeInfo.name].count + repeatCount;
    else
        queue[recipeInfo.name] = {recipeId = recipeId, count = repeatCount};
    end
    
    calculateQueue();    
end

-- ACTIONS

function CraftBar_UpdateQueueFrame()
        
    if (TradeSkillFrame and TradeSkillFrame:IsShown()) then
        calculateQueue();
    end
end

function CraftBar_ToggleQueueFrame()
    if (TradeSkillFrame.queueFrame:IsShown()) then
        TradeSkillFrame.queueFrame:Hide();
    else
        TradeSkillFrame.queueFrame:Show();
    
        calculateQueue(); 
    end
end

function CrafBar_AddToQueue()
    local recipeId = TradeSkillFrame.RecipeList:GetSelectedRecipeID();

    if (recipeId) then
        -- how much
        local repeatCount = 1;
    
        if (TradeSkillInputBox) then
            repeatCount = TradeSkillInputBox:GetNumber();
        end
    
        addToQueue(recipeId, repeatCount);
    end
end

local function removeFromQueue(recipeId, count)
    count = count or 1;
    
    local queue = getCreateQueue();
    
    for name, data in pairs(queue) do
        if (data.recipeId == recipeId) then
            if (IsAltKeyDown() or removeAll) then
                data.count = 0;
            else
                data.count = data.count - count;
            end
            
            if (data.count <= 0) then
                queue[name] = nil;
                selectedIndex = nil;
                TradeSkillFrame.queueFrame.scrollFrame.highlightFrame:Hide();
                TradeSkillFrame.queueFrame.removeButton:SetEnabled(false);
            end
            
            calculateQueue();
            return;
        end
    end
end

function CrafBar_RemoveFromQueue()
    if (selectedIndex) then
        local entry = entries[selectedIndex];
        
        if (entry and entry.recipeId) then
            removeFromQueue(entry.recipeId);
        end
    end
end

function CrafBar_ClearQueue()
    local tradeSkillID, skillLineName, skillLineRank, skillLineMaxRank, skillLineModifier = C_TradeSkillUI.GetTradeSkillLine();
    CraftBarQueue[skillLineName] = {}; 
    
    calculateQueue(); 
end

function CraftBarQueueEntry_OnEnter(self)
    if (self and self.data and self.data.itemLink) then
        GameTooltip_SetDefaultAnchor(GameTooltip, self);
        GameTooltip:SetOwner(self, "ANCHOR_RIGHT");

        GameTooltip:SetHyperlink(self.data.itemLink);
    end
end

function CraftBarQueueEntry_OnLeave(self)
    GameTooltip:FadeOut();
end

function CraftBarQueueEntry_OnClick(self, button, down)
    PlaySound("ACTIONBARBUTTONDOWN");
    selectedIndex = self.index;
    CleanUI_BattlePetGroups_OnUpdate();
    
    if (selectedIndex) then
        local entry = entries[selectedIndex];
        TradeSkillFrame.queueFrame.removeButton:SetEnabled(entry.recipeId ~= nil);
    end    
end

function CraftBarQueueEntry_CreateClicked(self)
    if (self.type == "CREATE") then
        local recipeId = self.data.recipeId;
        
        local recipeInfo = C_TradeSkillUI.GetRecipeInfo(recipeId);
        
        --weniger als gewünscht, wenn es nicht anders geht
        local count = min(recipeInfo.numAvailable, self.data.count);
        
        C_TradeSkillUI.CraftRecipe(recipeId, count);
        removeFromQueue(recipeId, count);
    elseif (self.type == "BUY") then
        BuyMerchantItem(self.index, self.buyQuantity);
    end
    
    CleanUI_BattlePetGroups_OnUpdate();
end